@@ -2,12 +2,16 @@
 SPARQL Update statements can be applied with :meth:`rdflib.graph.Graph.update`
 """
 
+from pathlib import Path
+
 import rdflib
 
-if __name__ == "__main__":
+EXAMPLES_DIR = Path(__file__).parent
 
+
+if __name__ == "__main__":
     g = rdflib.Graph()
-    g.load("foaf.n3", format="n3")
+    g.parse(f"{EXAMPLES_DIR / 'foaf.n3'}", format="n3")
 
     print(f"Initially there are {len(g)} triples in the graph")
 
@@ -15,8 +19,8 @@ if __name__ == "__main__":
         """
         PREFIX foaf: <http://xmlns.com/foaf/0.1/>
         PREFIX dbpedia: <http://dbpedia.org/resource/>
-        INSERT { 
-            ?s a dbpedia:Human . 
+        INSERT {
+            ?s a dbpedia:Human .
         }
         WHERE {
             ?s a foaf:Person .
