@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 This parser will interpret a JSON-LD document as an RDF Graph. See:
 
@@ -28,6 +27,7 @@ Example usage::
     True
 
 """
+
 # From: https://github.com/RDFLib/rdflib-jsonld/blob/feature/json-ld-1.1/rdflib_jsonld/parser.py
 
 # NOTE: This code reads the entire JSON object into memory before parsing, but
@@ -35,7 +35,7 @@ Example usage::
 from __future__ import annotations
 
 import warnings
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Union
 
 import rdflib.parser
 from rdflib.graph import ConjunctiveGraph, Graph
@@ -62,9 +62,11 @@ from ..shared.jsonld.keys import (
     VOCAB,
 )
 from ..shared.jsonld.util import (
+    _HAS_ORJSON,
     VOCAB_DELIMS,
     context_from_urlinputsource,
     json,
+    orjson,
     source_to_json,
 )
 
@@ -79,33 +81,83 @@ class JsonLDParser(rdflib.parser.Parser):
     def __init__(self):
         super(JsonLDParser, self).__init__()
 
-    def parse(self, source: InputSource, sink: Graph, **kwargs: Any) -> None:
-        # TODO: docstring w. args and return value
-        encoding = kwargs.get("encoding") or "utf-8"
+    def parse(
+        self,
+        source: InputSource,
+        sink: Graph,
+        version: float = 1.1,
+        skolemize: bool = False,
+        encoding: Optional[str] = "utf-8",
+        base: Optional[str] = None,
+        context: Optional[
+            Union[
+                List[Union[Dict[str, Any], str, None]],
+                Dict[str, Any],
+                str,
+            ]
+        ] = None,
+        generalized_rdf: Optional[bool] = False,
+        extract_all_scripts: Optional[bool] = False,
+        **kwargs: Any,
+    ) -> None:
+        """Parse JSON-LD from a source document.
+
+        The source document can be JSON or HTML with embedded JSON script
+        elements (type attribute = "application/ld+json"). To process as HTML
+        ``source.content_type`` must be set to "text/html" or
+        "application/xhtml+xml".
+
+        :param source: InputSource with JSON-formatted data (JSON or HTML)
+
+        :param sink: Graph to receive the parsed triples
+
+        :param version: parse as JSON-LD version, defaults to 1.1
+
+        :param encoding: character encoding of the JSON (should be "utf-8"
+            or "utf-16"), defaults to "utf-8"
+
+        :param base: JSON-LD `Base IRI <https://www.w3.org/TR/json-ld/#base-iri>`_, defaults to None
+
+        :param context: JSON-LD `Context <https://www.w3.org/TR/json-ld/#the-context>`_, defaults to None
+
+        :param generalized_rdf: parse as `Generalized RDF <https://www.w3.org/TR/json-ld/#relationship-to-rdf>`_, defaults to False
+
+        :param extract_all_scripts: if source is an HTML document then extract
+            all script elements, defaults to False (extract only the first
+            script element). This is ignored if ``source.system_id`` contains
+            a fragment identifier, in which case only the script element with
+            matching id attribute is extracted.
+
+        """
         if encoding not in ("utf-8", "utf-16"):
             warnings.warn(
                 "JSON should be encoded as unicode. "
                 "Given encoding was: %s" % encoding
             )
 
-        base = kwargs.get("base") or sink.absolutize(
-            source.getPublicId() or source.getSystemId() or ""
-        )
+        if not base:
+            base = sink.absolutize(source.getPublicId() or source.getSystemId() or "")
 
-        context_data = kwargs.get("context")
+        context_data = context
         if not context_data and hasattr(source, "url") and hasattr(source, "links"):
             if TYPE_CHECKING:
                 assert isinstance(source, URLInputSource)
             context_data = context_from_urlinputsource(source)
 
         try:
-            version = float(kwargs.get("version", "1.0"))
+            version = float(version)
         except ValueError:
-            version = None
+            version = 1.1
 
-        generalized_rdf = kwargs.get("generalized_rdf", False)
+        # Get the optional fragment identifier
+        try:
+            fragment_id = URIRef(source.getSystemId()).fragment
+        except Exception:
+            fragment_id = None
 
-        data = source_to_json(source)
+        data, html_base = source_to_json(source, fragment_id, extract_all_scripts)
+        if html_base is not None:
+            base = URIRef(html_base, base=base)
 
         # NOTE: A ConjunctiveGraph parses into a Graph sink, so no sink will be
         # context_aware. Keeping this check in case RDFLib is changed, or
@@ -116,32 +168,53 @@ class JsonLDParser(rdflib.parser.Parser):
         else:
             conj_sink = sink
 
-        to_rdf(data, conj_sink, base, context_data, version, generalized_rdf)
+        to_rdf(
+            data,
+            conj_sink,
+            base,
+            context_data,
+            version,
+            bool(generalized_rdf),
+            skolemize=skolemize,
+        )
 
 
 def to_rdf(
     data: Any,
     dataset: Graph,
     base: Optional[str] = None,
-    context_data: Optional[bool] = None,
+    context_data: Optional[
+        Union[
+            List[Union[Dict[str, Any], str, None]],
+            Dict[str, Any],
+            str,
+        ]
+    ] = None,
     version: Optional[float] = None,
     generalized_rdf: bool = False,
     allow_lists_of_lists: Optional[bool] = None,
+    skolemize: bool = False,
 ):
     # TODO: docstring w. args and return value
     context = Context(base=base, version=version)
     if context_data:
         context.load(context_data)
     parser = Parser(
-        generalized_rdf=generalized_rdf, allow_lists_of_lists=allow_lists_of_lists
+        generalized_rdf=generalized_rdf,
+        allow_lists_of_lists=allow_lists_of_lists,
+        skolemize=skolemize,
     )
     return parser.parse(data, context, dataset)
 
 
-class Parser(object):
+class Parser:
     def __init__(
-        self, generalized_rdf: bool = False, allow_lists_of_lists: Optional[bool] = None
+        self,
+        generalized_rdf: bool = False,
+        allow_lists_of_lists: Optional[bool] = None,
+        skolemize: bool = False,
     ):
+        self.skolemize = skolemize
         self.generalized_rdf = generalized_rdf
         self.allow_lists_of_lists = (
             allow_lists_of_lists
@@ -211,6 +284,8 @@ class Parser(object):
             subj = self._to_rdf_id(context, id_val)
         else:
             subj = BNode()
+            if self.skolemize:
+                subj = subj.skolemize()
 
         if subj is None:
             return None
@@ -281,7 +356,7 @@ class Parser(object):
             if term.type == JSON:
                 obj_nodes = [self._to_typed_json_value(obj)]
             elif LIST in term.container:
-                obj_nodes = [{LIST: obj_nodes}]
+                obj_nodes = [self._expand_nested_list(obj_nodes)]
             elif isinstance(obj, dict):
                 obj_nodes = self._parse_container(context, term, obj)
         else:
@@ -333,17 +408,21 @@ class Parser(object):
 
         context = context.get_context_for_term(term)
 
-        flattened = []
-        for obj in obj_nodes:
-            if isinstance(obj, dict):
-                objs = context.get_set(obj)
-                if objs is not None:
-                    obj = objs
-            if isinstance(obj, list):
-                flattened += obj
-                continue
-            flattened.append(obj)
-        obj_nodes = flattened
+        # Flatten deep nested lists
+        def flatten(n: Iterable[Any]) -> List[Any]:
+            flattened = []
+            for obj in n:
+                if isinstance(obj, dict):
+                    objs = context.get_set(obj)
+                    if objs is not None:
+                        obj = objs
+                if isinstance(obj, list):
+                    flattened += flatten(obj)
+                    continue
+                flattened.append(obj)
+            return flattened
+
+        obj_nodes = flatten(obj_nodes)
 
         if not pred_uri:
             return
@@ -357,6 +436,8 @@ class Parser(object):
             if not self.generalized_rdf:
                 return
             pred = BNode(bid)
+            if self.skolemize:
+                pred = pred.skolemize()
         else:
             pred = URIRef(pred_uri)
 
@@ -388,11 +469,11 @@ class Parser(object):
 
         if v11 and GRAPH in term.container and ID in term.container:
             return [
-                dict({GRAPH: o})
-                if k in context.get_keys(NONE)
-                else dict({ID: k, GRAPH: o})
-                if isinstance(o, dict)
-                else o
+                (
+                    dict({GRAPH: o})
+                    if k in context.get_keys(NONE)
+                    else dict({ID: k, GRAPH: o}) if isinstance(o, dict) else o
+                )
                 for k, o in obj.items()
             ]
 
@@ -404,23 +485,29 @@ class Parser(object):
 
         elif v11 and ID in term.container:
             return [
-                dict({ID: k}, **o)
-                if isinstance(o, dict) and k not in context.get_keys(NONE)
-                else o
+                (
+                    dict({ID: k}, **o)
+                    if isinstance(o, dict) and k not in context.get_keys(NONE)
+                    else o
+                )
                 for k, o in obj.items()
             ]
 
         elif v11 and TYPE in term.container:
             return [
-                self._add_type(
-                    context,
-                    {ID: context.expand(o) if term.type == VOCAB else o}
-                    if isinstance(o, str)
-                    else o,
-                    k,
+                (
+                    self._add_type(
+                        context,
+                        (
+                            {ID: context.expand(o) if term.type == VOCAB else o}
+                            if isinstance(o, str)
+                            else o
+                        ),
+                        k,
+                    )
+                    if isinstance(o, (dict, str)) and k not in context.get_keys(NONE)
+                    else o
                 )
-                if isinstance(o, (dict, str)) and k not in context.get_keys(NONE)
-                else o
                 for k, o in obj.items()
             ]
 
@@ -534,7 +621,10 @@ class Parser(object):
     def _to_rdf_id(self, context: Context, id_val: str) -> Optional[IdentifiedNode]:
         bid = self._get_bnodeid(id_val)
         if bid:
-            return BNode(bid)
+            b = BNode(bid)
+            if self.skolemize:
+                return b.skolemize()
+            return b
         else:
             uri = context.resolve(id_val)
             if not self.generalized_rdf and ":" not in uri:
@@ -559,7 +649,11 @@ class Parser(object):
         if not isinstance(node_list, list):
             node_list = [node_list]
 
-        first_subj = BNode()
+        first_subj: Union[URIRef, BNode] = BNode()
+        if self.skolemize and isinstance(first_subj, BNode):
+            first_subj = first_subj.skolemize()
+
+        rest: Union[URIRef, BNode, None]
         subj, rest = first_subj, None
 
         for node in node_list:
@@ -578,6 +672,8 @@ class Parser(object):
 
             graph.add((subj, RDF.first, obj))
             rest = BNode()
+            if self.skolemize and isinstance(rest, BNode):
+                rest = rest.skolemize()
 
         if rest:
             graph.add((subj, RDF.rest, RDF.nil))
@@ -587,9 +683,23 @@ class Parser(object):
 
     @staticmethod
     def _to_typed_json_value(value: Any) -> Dict[str, str]:
-        return {
-            TYPE: URIRef("%sJSON" % str(RDF)),
-            VALUE: json.dumps(
+        if _HAS_ORJSON:
+            val_string: str = orjson.dumps(
+                value,
+                option=orjson.OPT_SORT_KEYS | orjson.OPT_NON_STR_KEYS,
+            ).decode("utf-8")
+        else:
+            val_string = json.dumps(
                 value, separators=(",", ":"), sort_keys=True, ensure_ascii=False
-            ),
+            )
+        return {
+            TYPE: RDF.JSON,
+            VALUE: val_string,
         }
+
+    @classmethod
+    def _expand_nested_list(cls, obj_nodes: List[Any]) -> Dict[str, List[Any]]:
+        result = [
+            cls._expand_nested_list(o) if isinstance(o, list) else o for o in obj_nodes
+        ]
+        return {LIST: result}
