@@ -10,18 +10,24 @@ See also rdflib.plugin
 
 """
 
-from typing import IO, TYPE_CHECKING, Optional
+from __future__ import annotations
+
+from typing import IO, TYPE_CHECKING, Any, Optional, TypeVar, Union
+
 from rdflib.term import URIRef
 
 if TYPE_CHECKING:
     from rdflib.graph import Graph
 
+
 __all__ = ["Serializer"]
 
+_StrT = TypeVar("_StrT", bound=str)
+
 
 class Serializer:
-    def __init__(self, store: "Graph"):
-        self.store: "Graph" = store
+    def __init__(self, store: Graph):
+        self.store: Graph = store
         self.encoding: str = "utf-8"
         self.base: Optional[str] = None
 
@@ -30,12 +36,13 @@ class Serializer:
         stream: IO[bytes],
         base: Optional[str] = None,
         encoding: Optional[str] = None,
-        **args,
+        **args: Any,
     ) -> None:
         """Abstract method"""
 
-    def relativize(self, uri: str):
+    def relativize(self, uri: _StrT) -> Union[_StrT, URIRef]:
         base = self.base
         if base is not None and uri.startswith(base):
-            uri = URIRef(uri.replace(base, "", 1))
+            # type error: Incompatible types in assignment (expression has type "str", variable has type "Node")
+            uri = URIRef(uri.replace(base, "", 1))  # type: ignore[assignment]
         return uri
