@@ -1,24 +1,14 @@
 from __future__ import annotations
 
 import logging
-import sys
 from contextlib import ExitStack
-from pathlib import Path
-from test.utils.outcome import ExceptionChecker, OutcomeChecker, OutcomePrimitive
 from typing import TYPE_CHECKING, Any, Dict, Mapping, Optional, Set, Tuple, Type, Union
 
 import pytest
 
+from rdflib import Graph
 from rdflib.graph import Dataset
-from rdflib.term import URIRef
-
-if TYPE_CHECKING:
-    from rdflib._type_checking import _NamespaceSetString
-
-
-sys.path.append(str(Path(__file__).parent.parent.absolute()))
-from rdflib import Graph  # noqa: E402
-from rdflib.namespace import (  # noqa: E402
+from rdflib.namespace import (
     _NAMESPACE_PREFIXES_CORE,
     _NAMESPACE_PREFIXES_RDFLIB,
     OWL,
@@ -26,6 +16,11 @@ from rdflib.namespace import (  # noqa: E402
     Namespace,
     NamespaceManager,
 )
+from rdflib.term import URIRef
+from test.utils.outcome import ExceptionChecker, OutcomeChecker, OutcomePrimitive
+
+if TYPE_CHECKING:
+    from rdflib._type_checking import _NamespaceSetString
 
 
 def test_core_prefixes_bound():
