@@ -1,8 +1,7 @@
+from __future__ import annotations
+
 import json
-from test.utils import helper
-from test.utils.http import MethodName, MockHTTPResponse
-from test.utils.httpservermock import ServedBaseHTTPServerMock
-from test.utils.outcome import OutcomeChecker
+from http.client import IncompleteRead, RemoteDisconnected
 from typing import Dict, FrozenSet, List, Mapping, Sequence, Tuple, Type, Union
 
 import pytest
@@ -10,6 +9,10 @@ import pytest
 from rdflib import Graph, Literal, URIRef, Variable
 from rdflib.namespace import XSD
 from rdflib.term import BNode, Identifier
+from test.utils import helper
+from test.utils.http import MethodName, MockHTTPResponse
+from test.utils.httpservermock import ServedBaseHTTPServerMock
+from test.utils.outcome import OutcomeChecker
 
 
 @pytest.mark.webtest
@@ -26,7 +29,10 @@ def test_service():
         <http://www.w3.org/2000/01/rdf-schema#comment> ?dbpComment .
 
     } }  } limit 2"""
-    results = helper.query_with_retry(g, q)
+    try:
+        results = helper.query_with_retry(g, q)
+    except (RemoteDisconnected, IncompleteRead):
+        pytest.skip("this test uses dbpedia which is down sometimes")
     print(results.vars)
     print(results.bindings)
     assert len(results) == 2
@@ -51,7 +57,10 @@ def test_service_with_bind():
         <http://purl.org/dc/terms/subject> ?subject .
 
     } }  } limit 2"""
-    results = helper.query_with_retry(g, q)
+    try:
+        results = helper.query_with_retry(g, q)
+    except (RemoteDisconnected, IncompleteRead):
+        pytest.skip("this test uses dbpedia which is down sometimes")
     assert len(results) == 2
 
     for r in results:
@@ -87,7 +96,10 @@ def test_service_with_bound_solutions():
         }
         LIMIT 2
         """
-    results = helper.query_with_retry(g, q)
+    try:
+        results = helper.query_with_retry(g, q)
+    except (RemoteDisconnected, IncompleteRead):
+        pytest.skip("this test uses dbpedia which is down sometimes")
     assert len(results) == 2
 
     for r in results:
@@ -110,7 +122,10 @@ def test_service_with_values():
         <http://purl.org/dc/terms/subject> ?subject .
 
     } }  } limit 2"""
-    results = helper.query_with_retry(g, q)
+    try:
+        results = helper.query_with_retry(g, q)
+    except (RemoteDisconnected, IncompleteRead):
+        pytest.skip("this test uses dbpedia which is down sometimes")
     assert len(results) == 2
 
     for r in results:
@@ -127,7 +142,10 @@ def test_service_with_implicit_select():
     {
       values (?s ?p ?o) {(<http://example.org/a> <http://example.org/b> 1) (<http://example.org/a> <http://example.org/b> 2)}
     }} limit 2"""
-    results = helper.query_with_retry(g, q)
+    try:
+        results = helper.query_with_retry(g, q)
+    except (RemoteDisconnected, IncompleteRead):
+        pytest.skip("this test uses dbpedia which is down sometimes")
     assert len(results) == 2
 
     for r in results:
@@ -145,7 +163,10 @@ def test_service_with_implicit_select_and_prefix():
     {
       values (?s ?p ?o) {(ex:a ex:b 1) (<http://example.org/a> <http://example.org/b> 2)}
     }} limit 2"""
-    results = helper.query_with_retry(g, q)
+    try:
+        results = helper.query_with_retry(g, q)
+    except (RemoteDisconnected, IncompleteRead):
+        pytest.skip("this test uses dbpedia which is down sometimes")
     assert len(results) == 2
 
     for r in results:
@@ -163,7 +184,10 @@ def test_service_with_implicit_select_and_base():
     {
       values (?s ?p ?o) {(<a> <b> 1) (<a> <b> 2)}
     }} limit 2"""
-    results = helper.query_with_retry(g, q)
+    try:
+        results = helper.query_with_retry(g, q)
+    except (RemoteDisconnected, IncompleteRead):
+        pytest.skip("this test uses dbpedia which is down sometimes")
     assert len(results) == 2
 
     for r in results:
@@ -181,7 +205,10 @@ def test_service_with_implicit_select_and_allcaps():
         ?s <http://www.w3.org/2002/07/owl#sameAs> ?sameAs .
       }
     } LIMIT 3"""
-    results = helper.query_with_retry(g, q)
+    try:
+        results = helper.query_with_retry(g, q)
+    except (RemoteDisconnected, IncompleteRead):
+        pytest.skip("this test uses dbpedia which is down sometimes")
     assert len(results) == 3
 
 
@@ -208,7 +235,10 @@ WHERE {
         VALUES (?s ?p ?o) {(<http://example.org/a> <http://example.org/b> "c")}
     }
 }"""
-    results = helper.query_with_retry(g, q)
+    try:
+        results = helper.query_with_retry(g, q)
+    except (RemoteDisconnected, IncompleteRead):
+        pytest.skip("this test uses dbpedia which is down sometimes")
     assert results.bindings[0].get(Variable("o")) == Literal("c")
 
 
@@ -236,7 +266,10 @@ WHERE {
     }
     FILTER( ?o IN (<http://example.org/URI>, "Simple Literal", "String Literal"^^xsd:string, "String Language"@en) )
 }"""
-    results = helper.query_with_retry(g, q)
+    try:
+        results = helper.query_with_retry(g, q)
+    except (RemoteDisconnected, IncompleteRead):
+        pytest.skip("this test uses dbpedia which is down sometimes")
 
     expected = freeze_bindings(
         [
